# PHANTASM: <ins>PH</ins>ylogenomic <ins>AN</ins>alyses for the <ins>TA</ins>xonomy and <ins>S</ins>ystematics of <ins>M</ins>icrobes
## Developed by Joseph S. Wirth

## Source code is available at https://github.com/dr-joe-wirth/phantasm

## If you use our software, please cite our paper
**Automating microbial taxonomy workflows with PHANTASM: PHylogenomic ANalyses for the TAxonomy and Systematics of Microbes**

Joseph S. Wirth & Eliot C. Bush, 2023

doi: [10.1093/nar/gkad196](https://doi.org/10.1093/nar/gkad196)
‎
##### Note: PHANTASM requires an email address for communicating with NCBI as described in python's [Bio.Entrez](https://biopython.org/docs/latest/api/Bio.Entrez.html) package. The email address is not stored or used for any other purposes.
##### Note: PHANTASM does not always run properly with the current version of `podman` (3.4.7). It is unclear what is causing this issue.
‎
 
 
# Table of Contents
1. [Mounting the Docker image](#1-mounting-the-docker-image)

    1.1.[Running Docker as root](#11-running-docker-as-root)

    1.2 [Preparing Docker settings](#12-preparing-docker-settings)

    1.3 [Mounting the image as a container](#13-mounting-the-image-as-a-container)

    1.4 [Using Singularity instead of Docker](#14-using-singularity-instead-of-docker)

2. [Getting started with PHANTASM](#2-getting-started-with-phantasm)

    2.1. [Getting help](#21-getting-help)

    2.2. [Excluding specific taxa from phylogenomic analyses (optional)](#22-excluding-specific-taxa-from-phylogenomic-analyses-optional)

3. [Running PHANTASM](#3-running-phantasm)

    3.1. [Option 1: unknown reference genomes and unknown phylogenetic marker(s)](#31-option-1-unknown-reference-genomes-and-unknown-phylogenetic-markers)

    3.2. [Option 2: unknown reference genomes and known phylogenetic marker(s)](#32-option-2-unknown-reference-genomes-and-known-phylogenetic-markers)

    3.3. [Option 3: known reference genomes](#33-option-3-known-reference-genomes)

    3.4. [Option 4: ranking possible phylogenetic markers in a known set of genomes](#34-option-4-ranking-possible-phylogenetic-markers-in-a-known-set-of-genomes)

4. [Analyzing the results](#4-analyzing-the-results)

5. [Detailed descriptions of the workflows](#5-detailed-descriptions-of-the-workflows)

    5.1. [Option 1: Identifying suitable phylogenetic markers for your input genome](#51-option-1-identifying-suitable-phylogenetic-markers-for-your-input-genome)

    5.2. [Option 1: Refining the phylogeny and performing phylogenomic analyses](#52-option-1-refining-the-phylogeny-and-performing-phylogenomic-analyses)

    5.3. [Option 2: Using a known phylogenetic marker(s)](#53-option-2-using-a-known-phylogenetic-markers)

    5.4. [Option 3: Analyzing a set of user-specified genomes](#54-option-3-analyzing-a-set-of-user-specified-genomes)

    5.5. [Option 4: Ranking possible phylogenetic markers in a set of user-specified genomes](#55-option-4-ranking-possible-phylogenetic-markers-in-a-set-of-user-specified-genomes)

6. [Common error messages](#6-common-error-messages)

    6.1. [Problems with 16S rRNA gene sequence annotation (or lack thereof) in your input genome(s)](#61-problems-with-16s-rrna-gene-sequence-annotation-or-lack-thereof-in-your-input-genomes)

    6.2. [Warnings about LC variables when using Singularity](#62-warnings-about-lc_-variables-when-using-singularity)

# 1. Mounting the Docker image
## 1.1. Running Docker as `root`
Docker must be run as `root`. This should not be a problem if [Docker Desktop](https://www.docker.com/products/docker-desktop/) is installed on your computer. However, running with the command line may require the use of `sudo` (`su` to switch to `root`). To circumvent this issue, add yourself to the group `docker` as described [here](https://docs.docker.com/engine/install/linux-postinstall/):

    sudo groupadd docker
    sudo usermod -aG docker $USER

Performing these steps will allow Docker to run as `root` without requiring `sudo` or `su`. If you are unable to be a sudoer, you might consider using Singularity instead (see [section 1.4](#14-using-singularity-instead-of-docker))

## 1.2. Preparing Docker settings
Be sure to allocate at least 8gb of memory to docker.
  * If you cannot allocate 8gb, then you will need to reduce the number of taxa (see [section 2.2](#22-modifying-phantasms-settings-optional))
  * If you see the following error, then you have not allocated enough memory:

        Calculating core genes ... Killed

Be sure to allocate more processors to docker if you wish to take advantage of parallel processing
  * This is optional. PHANTASM can run successfully with only 1 CPU, but will run significantly faster with more.
  * The number of processors can be modified with the `-N` or `--num_threads` flag.

## 1.3. Mounting the image as a container:
Pull the docker image from docker hub

    docker pull jwirth/phantasm:latest

Make a working directory containing an input gbff file or a directory containing multiple gbff files (note: all gbff files MUST be annotated!). For example:

    mkdir ~/myworkdir
    cp my_genome.gbff ~/myworkdir

Run the container with working directory mounted as a volume within it (named `/data` below). You must provide the __ABSOLUTE PATH__ to your working directory. The mounted volume __MUST__ be named `/data`

    $ cd ~/myworkdir
    $ docker -v $(pwd):/data jwirth/phantasm:latest phantasm

## 1.4. Using Singularity instead of Docker
If you are unable to use Docker (eg. your server admin prohibits Docker due to issues with root access), then you can use Singularity instead. Instructions for installing and using Singularity can be found on [their website](https://docs.sylabs.io/guides/latest/user-guide/). Once installed, pull the image with the following command:

    $ singularity pull docker://jwirth/phantasm:latest

which will generate the file `phantasm_latest.sif`.

When using Singularity, you will likely see the following warning message:

    During startup - Warning messages:
    1: Setting LC_COLLATE failed, using "C"
    2: Setting LC_TIME failed, using "C"
    3: Setting LC_MESSAGES failed, using "C"
    4: Setting LC_MONETARY failed, using "C"
    5: Setting LC_PAPER failed, using "C"
    6: Setting LC_MEASUREMENT failed, using "C"

This is normal and can be safely ignored.

# 2. Getting started with PHANTASM
## 2.1. Getting help:
To find out which version of phantasm is running, use one of the following commands:

    docker run jwirth/phantasm:latest phantasm version

    singularity exec phantasm_latest.sif phantasm version

In order to get a help message, use one of the following commands:

    docker run jwirth/phantasm:latest phantasm help

    singularity exec phantasm_latest.sif phantasm help

Additional help on any given task (see [section 3](#3-running-phantasm) for available tasks) can be obtained with one of the following commands:

    docker run jwirth/phantasm:latest phantasm <task> --help

    singularity exec phantasm_latest.sif phantasm <task> --help


## 2.2. Excluding specific taxa from phylogenomic analyses (optional)
It is possible to exclude specific taxa from the refined phylogeny using the flag `-E` or `--exclude_taxa`. To do so, create a file containing exactly one NCBI Taxonomy id per line in the directory where you call PHANTASM. The excluded ids can represent any taxonomic rank (eg. species, genus, family, etc.). This feature is only relevant when using [option 1](#31-option-1-unknown-reference-genomes-and-unknown-phylogenetic-markers) or [option 2](#32-option-2-unknown-reference-genomes-and-known-phylogenetic-markers). This feature is largely experimental and should be used with caution.

# 3. Running PHANTASM
There are three ways of running PHANTASM. As show in the table below, the best option to use largely depends on the answer to two questions: 1) do you know a suitable set of genomes? and 2) do you know a suitable phylogenetic marker(s) for your genome(s) of interest?

     _____________________________________________________________________
    | reference genomes? | phylogenetic marker(s)? | best PHANTASM option |
    |--------------------|-------------------------|----------------------|
    |         no         |           no            |       option 1       |
    |         no         |           yes           |       option 2       |
    |         yes        |        irrelevant       |       option 3       |
    |____________________|_________________________|______________________|


There is also a [fourth option](#34-option-4-ranking-possible-phylogenetic-markers-in-a-known-set-of-genomes) that can be used identify phylogenetic markers for a set of user-specified of genomes. This option can only be used after running [option 2](#32-option-2-unknown-reference-genomes-and-known-phylogenetic-markers) or [option 3](#33-option-3-known-reference-genomes).

## 3.1. Option 1: unknown reference genomes and unknown phylogenetic marker(s)
If you have an annotated genome sequence but you do not know a suitable phylogenetic marker and you do not know which reference genomes would be appropriate to use, then PHANTASM can be run in three steps:

1. PHANTASM uses 16S rRNA gene sequences to estimate a taxonomic placement, then prioritizes taxonomic breadth in order to find genes that coevolve with the input genome(s) and its relatives.
2. The user looks at the results of this analysis and selects a suitable phylogenetic marker(s).
3. Using the selected marker(s), PHANTASM automatically selects a set of reference genomes and uses them to perform phylogenomic analyses.

### Optional: Bypassing the requirement for annotated 16S rRNA gene sequences in your input genome(s)

By default, PHANTASM will attempt to extract the 16S rRNA gene sequence(s) from your input genome(s) by relying on its accurate annotation in your input genome. It searches this gene with blastn against the bacterial and archaeal 16S sequences from [NCBI's Targetted Loci database](https://www.ncbi.nlm.nih.gov/refseq/targetedloci/). The blast results are then used to approximate the taxonomic placement of the input genome(s).

Annotated 16S rRNA gene sequences are not always available. It is possible for users to bypass this requirement by providing a file with NCBI Taxonomy IDs for the taxa that the input genomes may be related to. The specified taxa can be any NCBI Taxonomy classification as long as it is of rank Family or below (ie. Species, Genus, and Family taxids are acceptable, but Order, Class, and Phylum are not).

In order to bypass the 16S rRNA gene sequence requirement, you must provide these NCBI Taxonomy IDs as a file using the `-S` or `--skip_16S` flags. The file must contain exactly one NCBI Taxonomy ID per line. The taxids provided by the user cannot exceed the rank Family. This methodology allows you to specify a collection of NCBI Taxonomy IDs that are somewhat related to the input genome(s). You are free to obtain these IDs in whatever method you best see fit. Ultimately, PHANTASM will use a suitable phylogenetic marker to refine the set of reference genomes.

Keep in mind that annotated 16S rRNA gene sequences are **not** required if a phylogenetic marker and/or suitable reference genomes are already known (see sections [3.2](#32-option-2-unknown-reference-genomes-and-known-phylogenetic-markers) and [3.3](#33-option-3-known-reference-genomes) for more information).


### Step 1: Identifying putative phylogenetic markers
Run one of the following commands. Be sure to replace `my_genome.gbff` and `email@address.org` with appropriate values. If using multiple genomes, replace `my_genome.gbff` with the name of the directory containing the genome files (this directory must be nested within the working directory).

    docker run -v $(pwd):/data jwirth/phantasm:latest phantasm getPhyloMarker -i my_genome.gbff -e email@address.org

    singularity exec phantasm_latest.sif phantasm getPhyloMarker -i ./my_genome.gbff -e email@address.org

### Optional flags that can be used with this step:

    -N, --num_threads <int>      number of processors to use                    [default: 1]
    -L, --max_leaves <int>       maximum number of leaves in the species tree   [default: 50]
    -F, --fewer_coregenes        limit the core genes to those with ≤5% gaps    [default: no limiting]
    -S, --ski6_16S <file>        specify related taxids instead of relying on 16S homology
    -E, --exclude_taxa <file>    specify taxids to exclude
    -h, --help                   print a help message


##### Note: this step will take some time, especially when performing blastp.
‎
### Step 2: Selecting phylogenetic marker(s)
 After PHANTASM finishes running, examine the file `initialAnalysis/putativePhylogeneticMarkers.txt` and determine which gene(s) you wish to use as the phylogenetic marker(s).  In this example, it will also exist locally at `~/myworkdir/initialAnalysis/putativePhylogeneticMarkers.txt`. Core genes are ranked by their cophenetic correlation coefficients ("r<sub>ccc</sub>") and are listed from highest r<sub>ccc</sub> to lowest r<sub>ccc</sub>. Record either the `locus_tag` or the `gene_num` for the gene you wish to use as a phylogenetic marker. __As a general rule, an r<sub>ccc</sub> value of at least 0.9 is recommended for any gene used as a phylogenetic marker.__

Here is an example of what the first few lines of `putativePhylogeneticMarkers.txt` looks like (tab-delimited):

    cophenetic_corr_coef    locus_tag       protein_len     gene_name       annotation                                                    gene_num
    0.989149519694591       SPO_RS03315     1188            dnaE            dnaE - DNA polymerase III subunit alpha                       43248
    0.987983296592394       SPO_RS17770     1379            rpoB            rpoB - DNA-directed RNA polymerase subunit beta               46112
    0.985668236693517       SPO_RS15845     732                             primosomal protein N'                                         45731
    0.985456649245075       SPO_RS10520     1151            mfd             mfd - transcription-repair coupling factor                    44689
    0.981628830738660       SPO_RS11250     972             uvrA            uvrA - excinuclease ABC subunit UvrA                          44830
    0.979106734126949       SPO_RS16055     776             clpA            clpA - ATP-dependent Clp protease ATP-binding subunit ClpA    45773
    0.978417303151171       SPO_RS10405     913             gyrA            gyrA - DNA gyrase subunit A                                   44666
    . . .
‎

### Step 3: Refining the initial phylogeny and perform phylogenomic analyses
If you are using a locus tag(s), then run one of the following commands. Be sure to replace `my_genome.gbff`, `<locus tag>`, and `email@address.org` with appropriate values:

    docker run -v $(pwd):/data jwirth/phantasm:latest phantasm refinePhylogeny -i my_genome.gbff -t <locus tag> -e email@address.org

    singularity exec phantasm_latest.sif phantasm refinePhylogeny -i my_genome.gbff -t <locus tag> -e email@address.org

PHANTASM can handle multiple phylogenetic markers and/or multiple input genomes. If using multiple markers, they should be a comma-separated list **without spaces**. If using multiple input genomes, specify a directory containing genbank files; the same number of phylogenetic markers must be specified for each input genome.

### Optional flags that can be specified with this step

    -N, --num_threads <int>      number of processors to use                    [default: 1]
    -L, --max_leaves <int>       maximum number of leaves in the species tree   [default: 50]
    -B, --bootstrap <int>        number of bootstraps to perform                [default: no bootstrapping]
    -F, --fewer_coregenes        limit the core genes to those with ≤5% gaps    [default: no limiting]
    -E, --exclude_taxa <file>    specify taxids to exclude
    -h, --help                   print a help message

#### Notes:
  * this step will take some time as there are many slow steps, in particular:
    * blastp
    * calculating AAI
    * calculating ANI
    * making species trees (especially if bootstraps are involved)
  * If using multiple genomes:
    * replace `my_genome.gbff` with a directory containing the genome files
    * list at least one gene per genome as a comma-separated list (**no spaces allowed**)
  * If using multiple phylogenetic markers
    * list each gene as a comma-separated list (**no spaces allowed**)
    * can be used in conjunction with multiple genomes, but the you must specify a gene for each genome

## 3.2. Option 2: unknown reference genomes and known phylogenetic marker(s)
If you have an annotated genome sequence and you know suitable phylogenetic markers, but you do not know which reference genomes would be appropriate to use, then you can run PHANTASM in a single step.

Run one of the following commands (be sure to replace `<locus tags>`, `my_genome.gbff`, and `email@address.org` with appropriate values):

    docker run -v $(pwd):/data jwirth/phantasm:latest phantasm knownPhyloMarker -i my_genome.gbff -t <locus tags> -e email@address.org

    singularity exec phantasm_latest.sif phantasm knownPhyloMarker -i my_genome.gbff -t <locus tags> -e email@address.org

### Optional flags that can be specified with this step

    -O, --out <dir>              output directory                               [default: './finalAnalysis']
    -N, --num_threads <int>      number of processors to use                    [default: 1]
    -L, --max_leaves <int>       maximum number of leaves in the species tree   [default: 50]
    -B, --bootstrap <int>        number of bootstraps to perform                [default: no bootstrapping]
    -F, --fewer_coregenes        limit the core genes to those with ≤5% gaps    [default: no limiting]
    -E, --exclude_taxa <file>    specify taxids to exclude
    -h, --help                   print a help message

The locus tag(s) provided must be present within the input genome(s) and indicate the phylogenetic marker(s) that you wish to use. PHANTASM can handle multiple phylogenetic markers and/or multiple input genomes. If using multiple markers, they should be a comma-separated list **without spaces**. If using multiple input genomes, specify a directory containing genbank files; the same number of phylogenetic markers must be specified for each input genome.

### Notes:
  * this step will take some time as there are many slow steps, in particular:
    * blastp
    * calculating AAI
    * calculating ANI
    * making species trees (especially if bootstraps are involved)
  * If using multiple genomes:
    * replace `my_genome.gbff` with a directory containing the genome files
    * list at least one gene per genome as a comma-separated list (**no spaces allowed**)
  * If using multiple phylogenetic markers
    * list each gene as a comma-separated list
    * can be used in conjunction with multiple genomes, but the you must specify the same number of genes for each genome

## 3.3. Option 3: known reference genomes
It is also possible to run PHANTASM on a set of user-specified genomes. This can be accomplished with a single command.

You must first create a "human map file". This file tells PHANTASM what the human-readable names are (used in the trees and the alignments) for each genome file. **It should be formatted with exactly two columns separated by a single tab.** The first column is the filename of the genbank file (not including the path), and the second column is the human readable name. **Important note:** _the desired outgroup must be listed **last** in the human map file_.  For example:

    GCF_009496005.1_ASM949600v1_genomic.gbff                                Tritonibacter_litoralis|SM1979|GCF_009496005
    GCF_007923355.1_ASM792335v1_genomic.gbff                                Phaeobacter_marinintestinus|UB-M7|GCF_007923355
    GCF_900106805.1_IMG-taxon_2693429872_annotated_assembly_genomic.gbff    Ruegeria_halocynthiae|DSM_27839_T|GCF_900106805
    GCF_000511355.1_ASM51135v1_genomic.gbff                                 Leisingera_methylohalidivorans|DSM_14336_T|GCF_000511355
    GCF_900172225.1_Tr.litoreus_CECT7639_Spades_Prokka_genomic.gbff         Ruegeria_litorea|CECT_7639_T|GCF_900172225
    GCF_900102795.1_IMG-taxon_2622736501_annotated_assembly_genomic.gbff    Epibacterium_ulvae|U95_T|GCF_900102795
    GCF_001458295.1_Ruegeria_sp._CECT5091_Spades_Prokka_genomic.gbff        Ruegeria_denitrificans|CECT_5091_T|GCF_001458295
    GCF_001507545.1_ASM150754v1_genomic.gbff                                Ruegeria_profundi|ZGT108_T|GCF_001507545
    GCF_009617595.1_ASM961759v1_genomic.gbff                                Tritonibacter_aquimaris|SM1969|GCF_009617595
    my_assembly.gbk                                                         Ruegeria_sp
    outgroup_genome_seq.gbff                                                outgroup_name

Once this file has been created, run one of the following command to analyze the provided genomes. Be sure to replace `input_genome_dir`, `human_map.txt`, and `email@address.org` with appropriate values:

    docker run -v $(pwd):/data jwirth/phantasm:latest phantasm analyzeGenomes -i input_genome_dir -m human_map.txt -e email@address.org

    singularity exec phantasm_latest.sif phantasm analyzeGenomes -i input_genome_dir -m human_map.txt -e email@address.org

### Optional flags that can be specified with this step

    -O, --out <dir>            output directory                               [default: './finalAnalysis']
    -N, --num_threads <int>    number of processors to use                    [default: 1]
    -B, --bootstrap <int>      number of bootstraps to perform                [default: no bootstrapping]
    -F, --fewer_coregenes      limit the core genes to those with ≤5% gaps    [default: no limiting]
    -h, --help                 print a help message

##### Note: the specified output directory must not already exist.

## 3.4. Option 4: ranking possible phylogenetic markers in a known set of genomes
This option is used to rank phylogenetic markers **after** running [option 2](#32-option-2-unknown-reference-genomes-and-known-phylogenetic-markers) or [option 3](#33-option-3-known-reference-genomes).

Run one of the following commands. Be sure to replace `./phantasm_results` with an appropriate value:

    docker run -v $(pwd):/data jwirth/phantasm:latest phantasm rankPhyloMarkers -i ./phantasm_results

    singularity exec phantasm_latest.sif phantasm rankPhyloMarkers -i ./phantasm_results

### Optional flags that can be specified with this step

    -O, --out <file>           output file                    [default: 'phantasm_results/putativePhylogeneticMarkers.txt']
    -N, --num_threads <int>    number of processors to use    [default: 1]
    -h, --help                 print a help message

[See above](#step-2-selecting-phylogenetic-markers) for details on interpretting the data in `putativePhylogeneticMarkers.txt`.

# 4. Analyzing the results
All data can be found within the directory where you called `phantasm`. Unless the flag `-O` or `--out` was used, the results will be found in the folder `finalAnalysis`.
  * The following files are likely to be the most useful for phylogenomic and taxonomic analyses:
    * `speciesTree.nwk`: this is the species tree, it is rooted on the outgroup, and the outgroup is present in the tree
    * `speciesTree_outgroupPruned.nwk`: this is the same species tree as above, but the outgroup has been pruned to allow better resolution of the relevant phylogenomic relationships
    * `aai_matrix.txt`: this is a raw text file containing the average amino acid identities for all of the taxa in `speciesTree_outgroupPruned.nwk`
    * `aai_heatmap.pdf`: this is a visualization of the data in `aai_matrix.txt` and the taxa are ordered to match the order of the taxa in `speciesTree_outgroupPruned.nwk`. Due to a bug in the R package `gplots`, the tree cannot be plotted alongside the heatmap at this time.
    * `ani_matrix.txt`: this is a raw text file containing the average nucleotide identities for all of the taxa in `speciesTree_outgroupPruned.nwk`
    * `ani_heatmap.pdf`: this is a visualization of the data in `ani_matrix.txt` and the taxa are ordered to match the order of the taxa in `speciesTree_outgroupPruned.nwk`. Due to a bug in the R package `gplots`, the tree cannot be plotted alongside the heatmap at this time.
    * `coreGenesSummary.txt`: this is a tab-delimited file containing detailed information on the core genes used to construct the species tree. Locus tags, gene numbers, gene names, and annotations are only specified for the input genomes. The indicated alignment file can be used to determine these data for other reference genomes used in the analysis.
    * `wgsHumanMap.txt`: this file lists all of the accession numbers for the genomes used in the analysis as well as their human readable names found in the alignments, trees, and heatmaps. The equivalent of this file was provided by the user if `analyzeGenomes` was used.

# 5. Detailed descriptions of the workflows

## 5.1. Option 1: Identifying suitable phylogenetic markers for your input genome
  * Extracts the 16S rRNA gene sequences from the input genbank.
  * Uses the 16S rRNA gene sequences and BLASTn to search [NCBI's Targetted Loci database](https://www.ncbi.nlm.nih.gov/refseq/targetedloci/) in order to determine which taxonomic orders are related to the input genome(s).
  * Retreives data from NCBI's Taxonomy database to create a single taxonomy encompassing all the identified orders.
  * Reconciles the NCBI-based taxonomy with LPSN's taxonomic structure.
  * Imports data from NCBI's Assembly database for the species represented in the taxonomy.
  * Uses the BLASTn results and the taxonomic structure to determine which taxa could serve as the ingroup/outgroup.
  * Downloads assemblies for the ingroup and the outgroup the number to download is specified in `<path to phatasm>/param.py` in the `MAX_LEAVES` variable.
  * Calculates core genes for the set of gbff files.
  * Creates a species tree based on a concatenated alignment of the core genes.
  * Creates an individual gene tree for each core gene.
  * Uses the species tree and gene trees to calculate a score of how well correlated each individual core gene is to the species tree's topology. Saves this information in the file `initialAnalysis/putativePhylogeneticMarkers.txt`.

## 5.2. Option 1: Refining the phylogeny and performing phylogenomic analyses
  * Using the phylogenetic marker(s) specified by the user, finds the most closely-related taxa to the input genome.
  * Uses this information to select genomes from:
  * many species from the closest genus/genera
  * several species from more distant genera
  * Downloads the assemblies for the new ingroup and the outgroup
  * Calculates the core genes for the refined set of genomes.
  * Creates a species tree based on a concatenated alignment of the core genes.
  * Calculates the average amino acid identity (AAI) for the species in the tree (excluding the outgroup).
  * Calculates the average nucleotide identity (ANI) for the species in the tree (excluding the outgroup).

## 5.3. Option 2: Using a known phylogenetic marker(s)
  * Using the phylogenetic marker(s) specified by the user, finds the most closely-related taxa to the input genome via BLASTp against either NCBI's nr or refseq_protein databases.
  * Uses the BLASTp results to determine which taxonomic orders are related to the input.
  * Retrieves data from NCBI's Taxonomy database to create a single taxonomy encompassing all the identified orders.
  * Reconciles the NCBI-based taxonomy with LPSN's taxonomic structure.
  * Imports data from NCBI's Assembly database for the species represented in the taxonomy.
  * Uses the BLASTp results and the taxonomic structure to determine which taxa could serve as the ingroup/outgroup.
  * Downloads assemblies for the ingroup and the outgroup the number to download is specified in `<path to phantasm>/param.py` in the `MAX_LEAVES` variable.
  * Calculates core genes for the set of gbff files.
  * Creates a species tree based on a concatenated alignment of the core genes.
  * Calculates the average amino acid identity (AAI) for the species in the tree (excluding the outgroup).
  * Calculates the average nucleotide identity (ANI) for the species in the tree (excluding the outgroup).

## 5.4. Option 3: Analyzing a set of user-specified genomes
  * Calculates core genes for the set of gbff files.
  * Creates a species tree based on a concatenated alignment of the core genes.
  * Calculates the average amino acid identity (AAI) for the species in the tree (excluding the outgroup).
  * Calculates the average nucleotide identity (ANI) for the species in the tree (excluding the outgroup).

## 5.5. Option 4: Ranking possible phylogenetic markers in a set of user-specified genomes
  * Calculates core genes for the set of gbff files.
  * Creates a species tree based on a concatenated alignment of the core genes.
  * Creates an individual gene tree for each core gene.
  * Uses the species tree and gene trees to calculate a score of how well correlated each individual core gene is to the species tree's topology. Saves this information in the file `putativePhylogeneticMarkers.txt`.

# 6. Common error messages
## 6.1. Problems with 16S rRNA gene sequence annotation (or lack thereof) in your input genome(s)
### Possible error messages
`BaseException: Could not extract 16S rRNA gene sequences from the provided genbank file.`

`RuntimeError: The file '/mydata/initialAnalysis/16S/query.16SrRNA.blastn' does not contain any valid blastn hits`

### Possible solutions
__Option 1.__ Create a file named `taxids.txt` as described in the [section 3.1](#optional-bypassing-the-requirement-for-annotated-16s-rrna-gene-sequences-in-your-input-genomes).

__Option 2.__ Use a known phylogenetic marker as described in [section 3.2](#32-option-2-unknown-reference-genomes-and-known-phylogenetic-markers).

## 6.2 Warnings about `LC_` variables when using Singularity
### Warning message

    During startup - Warning messages:
    1: Setting LC_COLLATE failed, using "C"
    2: Setting LC_TIME failed, using "C"
    3: Setting LC_MESSAGES failed, using "C"
    4: Setting LC_MONETARY failed, using "C"
    5: Setting LC_PAPER failed, using "C"
    6: Setting LC_MEASUREMENT failed, using "C"

### Solution:
This is expected behavior when using the Docker image with Singularity and can be safely ignored.
